import { getTextByPathList } from './utils'
import { getShadow } from './shadow'
import { getFillType, getSolidFill } from './fill'

export function getFontType(node, type, warpObj) {
  let typeface = getTextByPathList(node, ['a:rPr', 'a:latin', 'attrs', 'typeface'])

  if (!typeface) {
    const fontSchemeNode = getTextByPathList(warpObj['themeContent'], ['a:theme', 'a:themeElements', 'a:fontScheme'])

    if (type === 'title' || type === 'subTitle' || type === 'ctrTitle') {
      typeface = getTextByPathList(fontSchemeNode, ['a:majorFont', 'a:latin', 'attrs', 'typeface'])
    } 
    else if (type === 'body') {
      typeface = getTextByPathList(fontSchemeNode, ['a:minorFont', 'a:latin', 'attrs', 'typeface'])
    } 
    else {
      typeface = getTextByPathList(fontSchemeNode, ['a:minorFont', 'a:latin', 'attrs', 'typeface'])
    }
  }

  return typeface || ''
}

export function getFontColor(node, pNode, lstStyle, pFontStyle, lvl, warpObj) {
  const rPrNode = getTextByPathList(node, ['a:rPr'])
  let filTyp, color
  if (rPrNode) {
    filTyp = getFillType(rPrNode)
    if (filTyp === 'SOLID_FILL') {
      const solidFillNode = rPrNode['a:solidFill']
      color = getSolidFill(solidFillNode, undefined, undefined, warpObj)
    }
  }
  if (!color && getTextByPathList(lstStyle, ['a:lvl' + lvl + 'pPr', 'a:defRPr'])) {
    const lstStyledefRPr = getTextByPathList(lstStyle, ['a:lvl' + lvl + 'pPr', 'a:defRPr'])
    filTyp = getFillType(lstStyledefRPr)
    if (filTyp === 'SOLID_FILL') {
      const solidFillNode = lstStyledefRPr['a:solidFill']
      color = getSolidFill(solidFillNode, undefined, undefined, warpObj)
    }
  }
  if (!color) {
    const sPstyle = getTextByPathList(pNode, ['p:style', 'a:fontRef'])
    if (sPstyle) color = getSolidFill(sPstyle, undefined, undefined, warpObj)
    if (!color && pFontStyle) color = getSolidFill(pFontStyle, undefined, undefined, warpObj)
  }
  return color || ''
}

export function getFontSize(node, slideLayoutSpNode, type, slideMasterTextStyles) {
  
  let fontSize

  // 检查列表样式中的字体大小
  const lstStyleSz = getTextByPathList(node, ['a:lstStyle', 'a:lvl1pPr', 'a:defRPr', 'attrs', 'sz'])
  if (lstStyleSz) {
    fontSize = parseInt(lstStyleSz) / 100
  }

  // 检查直接在文本上设置的字体大小
  if (!fontSize || isNaN(fontSize)) {
    const directSz = getTextByPathList(node, ['a:rPr', 'attrs', 'sz'])
    if (directSz) {
      fontSize = parseInt(directSz) / 100
    }
  }

  // 检查段落级别的字体大小
  if (!fontSize || isNaN(fontSize)) {
    const pPrNode = getTextByPathList(node, ['a:pPr'])
    if (pPrNode) {
      const lvl = parseInt(pPrNode.attrs?.lvl || '0')
      
      // 检查布局中的字体大小设置
      const layoutSz = getTextByPathList(slideLayoutSpNode, ['p:txBody', 'a:lstStyle', `a:lvl${lvl + 1}pPr`, 'a:defRPr', 'attrs', 'sz'])
      if (layoutSz) {
        fontSize = parseInt(layoutSz) / 100
      }
      
      // 如果没有找到布局大小，检查母版样式
      if ((!fontSize || isNaN(fontSize)) && slideMasterTextStyles) {
        let styleNode
        if (type === 'title' || type === 'ctrTitle') {
          styleNode = getTextByPathList(slideMasterTextStyles, ['p:titleStyle'])
        }
        else if (type === 'body') {
          styleNode = getTextByPathList(slideMasterTextStyles, ['p:bodyStyle'])
        }
        else if (type === 'subTitle') {
          styleNode = getTextByPathList(slideMasterTextStyles, ['p:otherStyle'])
        }
        
        if (styleNode) {
          const masterSz = getTextByPathList(styleNode, [`a:lvl${lvl + 1}pPr`, 'a:defRPr', 'attrs', 'sz'])
          if (masterSz) {
            fontSize = parseInt(masterSz) / 100
          }
        }
      }
    }
  }

  // 4. 如果还是没有找到，使用默认大小
  if (!fontSize || isNaN(fontSize)) {
    if (type === 'title' || type === 'ctrTitle') {
      fontSize = 60 // 标题默认60磅
    }
    else if (type === 'subTitle') {
      fontSize = 32 // 副标题默认32磅
    }
    else if (type === 'body') {
      fontSize = 18 // 正文默认18磅
    }
    else {
      fontSize = 18 // 其他默认18磅
    }
  }

  return fontSize + 'pt'
}

export function getFontBold(node) {
  return getTextByPathList(node, ['a:rPr', 'attrs', 'b']) === '1' ? 'bold' : ''
}

export function getFontItalic(node) {
  return getTextByPathList(node, ['a:rPr', 'attrs', 'i']) === '1' ? 'italic' : ''
}

export function getFontDecoration(node) {
  return getTextByPathList(node, ['a:rPr', 'attrs', 'u']) === 'sng' ? 'underline' : ''
}

export function getFontDecorationLine(node) {
  return getTextByPathList(node, ['a:rPr', 'attrs', 'strike']) === 'sngStrike' ? 'line-through' : ''
}

export function getFontSpace(node) {
  const spc = getTextByPathList(node, ['a:rPr', 'attrs', 'spc'])
  return spc ? (parseInt(spc) / 100 + 'pt') : ''
}

export function getFontSubscript(node) {
  const baseline = getTextByPathList(node, ['a:rPr', 'attrs', 'baseline'])
  if (!baseline) return ''
  return parseInt(baseline) > 0 ? 'super' : 'sub'
}

export function getFontShadow(node, warpObj) {
  const txtShadow = getTextByPathList(node, ['a:rPr', 'a:effectLst', 'a:outerShdw'])
  if (txtShadow) {
    const shadow = getShadow(txtShadow, warpObj)
    if (shadow) {
      const { h, v, blur, color } = shadow
      if (!isNaN(v) && !isNaN(h)) {
        return h + 'pt ' + v + 'pt ' + (blur ? blur + 'pt' : '') + ' ' + color
      }
    }
  }
  return ''
}